/**
    @file adt7310.c

    @brief Library file for ADT7310
 *

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

/* INCLUDES ************************************************************************/
#include "adt7310.h"

/* CONSTANTS ***********************************************************************/
const uint8_t reset[4]    = {0xFF, 0xFF, 0xFF, 0xFF};
const uint8_t get_temp[3] = {0x50, 0xFF, 0xFF}; // command byte

/* GLOBAL VARIABLES ****************************************************************/

/* LOCAL VARIABLES *****************************************************************/
uint8_t din[4] = {0};

/* MACROS **************************************************************************/

/* LOCAL FUNCTIONS / INLINES *******************************************************/

/* FUNCTIONS ***********************************************************************/
void temperature_setup()
{
    // Configure the SPI MasterI
    SPIM_initialise_ints();
    // SPI Master Temp Sensor
    SPIM_configure_slave(IOMUX_OUT_SPI_MASTER_SS_N_0, // slave 0
    					 1, // Idle state high
                         SPI_BIT_ORDER_MSB, // send MSB first
                         48, // (48 MHz clock / 2) / 48 == 500 kHz SCLK
                         SPI_MODE_3, // CPOL=1, CPHA=1
                         0); // no delay between SS and SCLK
}

/**
 @brief      Reset ADT7310 temperature sensor
 @details    Sends a reset command to the ADT7310 temperature sensor using the
 	 	 	 SPI Master peripheral.
   	   	   	 Resets the contents of all registers to their power-on values.
 	 	 	 Following a reset, the user should allow a period of 500 us
 	 	 	 before addressing the SPI interface.
 **/
void temperature_reset()
{
    // A Logic 1 must be written to the ADT7310 line for at least 32
    // serial clock cycles to reset the serial interface.
	SPIM_SS_SETUP(1, 0);
    SPIM_SS_ON;
    SPIM_transceive_ints(reset,
                         din, //dummy response
                         4);
    SPIM_SS_OFF;
}

/**
 @brief      Read ADT7310 temperature sensor
 @details    Sends a read command to the ADT7310 temperature sensor using the
 	 	 	 SPI Master peripheral.
 @returns    Temperature in units of 0.01 C. Divide by 100 to obtain C.
 **/
int16_t temperature_read()
{
	int16_t s16;

	SPIM_SS_SETUP(1, 0);
    SPIM_SS_ON;
    SPIM_transceive_ints(get_temp, din, 3);
    SPIM_SS_OFF;

    // The result is in 13-bit mode and the conversion is in bits
    // 15:3 of the word received after the command is sent (MSB first).
    // The lower 4 bits of the conversion are fractions of a degree.
    s16 = (int16_t)(din[1] << 8 | din[2]);
    s16 = ((int32_t)s16 * 100) / (16*8);
    // Result is temperature in units of 0.01 C.
    return s16;
}