/**
    @file ft51_usb_initialise.c

    @brief USB Library API initialise function.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdlib.h>
#include <string.h>
#include "ft51.h"
#include "ft51_sfr.h"
#include "ft51_interrupts.h"
#include "ft51_io_registers.h"
#include "registers/ft51_registers.h"
#include "ft51_usb.h"
#include "ft51_usb_internal.h"

// Global across all USB library sources (extern in ft51_usb_private.h):

/**
 @name Hub configuration
 @brief Stores configuration (enabled/disabled, remote-wakeup etc.)
 specified in USB_initialise.
  */
uint8_t USB_hub_config = 0;
/**
 @name Current state of USB device
 @brief Powered, Default, Address, Configured or Suspended.
 */
USB_STATE USB_current_state = ATTACHED;
/**
 @name Suspend/resume status changed flag (counter).
 @brief Variable used to count USB suspend/resume status changes.
 When the USB processing routine is next called the suspend
 or resume callback routines will be called.
 */
volatile uint8_t USB_suspend_changed = 0;
/**
 @name Reset signal flag.
 @brief Variable used to request USB reset event to be actioned
 the next time the USB processing routine is called.
 */
volatile uint8_t USB_reset_signal = 0;
/**
 @name Core suspend request flag.
 @brief Variable used to request a core suspend to occur.
 This signals that a SYSTEM_STOP_REQUEST should be performed
 by the application when it is next possible. If this is
 set by a USB suspend event then it should be handled in less
 than 10ms to comply with the USB specifications.
 */
volatile int8_t USB_core_suspend_request = 0;
/**
 @name Link power management status changed flag.
 @brief Variable used to signal that a Link Power Management
 event has occurred.
 */
volatile uint8_t USB_LPM_changed = 0;

/**
 @name Current USB configuration value.
 @brief Zero when the device is in the Address state but non-
 zero once configured. Must match a valid configuration in the
 configuration descriptor. Set by SET_CONFIGURATION request.
 */
uint8_t USB_configuration = 0;

/** Not used... Flag set when remote wakeup called. **/
/*uint8_t                 USB_remote_wakeup;*/

/** Standard request callback **/
USB_request_callback USB_standard_req_cb = NULL;
/** Class request callback **/
USB_request_callback USB_class_req_cb = NULL;
/** Vendor request callback **/
USB_request_callback USB_vendor_req_cb = NULL;
/** Callback when the device resumes **/
USB_suspend_callback USB_resume_cb = NULL;
/** Callback when the device suspends **/
USB_suspend_callback USB_suspend_cb = NULL;
/** Callback when the host initiates a reset **/
USB_reset_callback USB_reset_cb = NULL;
/** Callback when the host makes a LPM (Link Power Management) change **/
USB_suspend_callback USB_lpm_cb = NULL;

/** Array of pointers to device endpoints. Force into large memory. **/
__xdata USB_endpoint USB_ep[USB_MAX_ENDPOINT_COUNT];
/** Decode of USB_ENDPOINT_SIZE definitions into actual endpoint
 *  Max Packet size values. **/
__code const uint16_t USB_ep_size_bytes[12] =
{ 8, 16, 32, 64, 96, 128, 160, 192, 256, 320, 384, 504 };

/** Array to hold any received SETUP tokens. **/
__xdata uint8_t USB_received_setup_packet[8];

void USB_initialise(USB_ctx *c)
{
    // Enable hub based on bitmap supplied (which matches the
    // USB_TOP_ENABLE register).  Note that the HUB_ENABLE bit
    // of that register is always read as 1, even if the hub is
    // disabled; so we keep a copy of the hub_en.value specified
    // by the caller, to use in later hub state calculations.
    USB_hub_config = (MASK_FT122_ENABLE | MASK_HUB_COMPD_DEV);
    USB_hub_config |= (c->hub_en.value);
    IO_REG_GENERAL_WRITE(TOP_USB_ENABLE, USB_hub_config);
    // Remote wakeup must be enabled by a Set Feature - default off.
    USB_clear_remote_wakeup();

    FT122_CMD = SET_MODE;
    FT122_DATA = 0x10;  // Enable DP pullup
    FT122_DATA = 0x43;  // CLOCKOUT = 48 MHz / (3 + 1) = 12 MHz
    //FT122_CMD = SET_LPM_ENABLE;  // 0xEE, USB 2.0 Link Power Management

    // Callbacks for responding to SETUP requests
    USB_standard_req_cb = c->standard_req;
    USB_class_req_cb = c->class_req;
    USB_vendor_req_cb = c->vendor_req;
    // Callbacks for handling bus status changes
    USB_suspend_cb = c->suspend;
    USB_resume_cb = c->resume;
    USB_reset_cb = c->reset;
    USB_lpm_cb = c->lpm;

    // Initialise (zero) all the endpoint pointers.
    memset(USB_ep, 0, sizeof(USB_ep));

    // Each device has, as a minimum , an IN and OUT control endpoint.
    // Control Out
    USB_create_endpoint(USB_EP_0, USB_EP_CTRL, USB_DIR_OUT, c->ep_size);
    // Control In
    USB_create_endpoint(USB_EP_0, USB_EP_CTRL, USB_DIR_IN, c->ep_size);

    USB_current_state = POWERED;
    //USB_configuration = 0;

    // Register for FT122 interrupts
    interrupts_register(USB_ISR, interrupts_FT122);
}